
#include "parsecmdline.hpp"
#include "reorder.hpp"
#include "pstools.hpp"
#include <list>
#include <iostream>
#include <boost/lexical_cast.hpp>

void printHelp()
{
        char* version=
	"pstome (Postscript Tome) version 0.94 - first release\n"
        "Copyright (c) Bernhard Gnther, 2006. See LICENSE for details.\n";

        char* help=
	"pstome (Postscript Tome) version 0.94 - first release\n"
        "Copyright (c) Bernhard Gnther, 2006. See LICENSE for details.\n"
        "Usage: pstome [-s|-d] [-p<number_of_pages>] [-r] <infile> [<outfile1>] [<outfile2>]\n"
        "\n"
        "   <infile> is either a postscript or a pdf file\n"
        "\n"
        "   -h,--help\tthis help\n"
        "   -s,--simlex\tproduce two files for simplex printing (default)\n"
        "   -d,--duplext\tproduce one file for duplex printing\n"
        "   -p<n>\tproduce n number of pages (must be multiple of 4)\n"
        "   -r,--reverse\treverse pages for backside-sheets (ignored in duplex)\n"
        "\n"
        "If mode is simplex and only one outfile is given, outfile1 will be used as\n"
        "prefix and files generated are <outfile1>-front.ps and <outfile1>-back.ps\n"
        "\n"
	"See pstome.pdf for a description what PS-Tome does.\n"
        "\n";

        std::cerr << help;

}

int main (int argc, char** argv)
{
        if (argc < 2){
                printHelp();
                return 0;
        }
        try {
                pstome::ParseCmdLine settings(argc, argv);
                pstome::PsTools processor(settings.infile);

		unsigned int pagecount=processor.getPageCount();
		if (pagecount==0){
			pagecount=settings.pnumber;
		}
		if (pagecount==0){
			std::cerr << "Unable to determine pagecount. Please specify number of pages (-p<n>)." << std::endl;
			return 253;
		} else {
			if ( (settings.pnumber > 0) && (settings.pnumber < pagecount) ) {
				pagecount = settings.pnumber;
			}
		}

                pstome::Reorder book(pagecount);
                std::list<unsigned int> front;
                std::list<unsigned int> back;
		if (settings.mode==pstome::SIMPLEX){
                	book.getSimplex(front,back,settings.reverse);
			processor.generatePostscript(settings.outfile1,front);
			processor.generatePostscript(settings.outfile2,back);
		} else if (settings.mode==pstome::DUPLEX){
                	book.getDuplex(front);
			processor.generatePostscript(settings.outfile1,front);
		}

        }
        catch (boost::bad_lexical_cast &){
                std::cerr << "Argument <" << argv[1] << "> is not a number." << std::endl;
                return 254;
        }
        catch (pstome::Exceptions ecode) {
                if (ecode==pstome::INCORRECTPAGENUMBER){
                        std::cerr << "Pagecount is not a multiple of 4." << std::endl;
                        return (252);
                }
                if (ecode==pstome::CMDLINEHELP){
                        printHelp();
                        return (0);
                }
                if (ecode==pstome::CMDLINEVERSION){
                        printHelp();
                        return (0);
                }
                if (ecode==pstome::CMDLINESIMPLEXDUPLEX){
                        std::cerr << "Simplex and Duplex not possible simultaneously. Try \"pstome -h\"." << std::endl;
                        return (251);
                }
                if (ecode==pstome::CMDLINENOINFILE){
                        std::cerr << "No input file. Try \"pstome -h\"." << std::endl;
                        return (250);
                }
                if (ecode==pstome::CMDLINENOOUTFILE){
                        std::cerr << "No output file. Try \"pstome -h\"." << std::endl;
                        return (249);
                }
                if (ecode==pstome::CMDLINEUNKNOWNOPTION){
                        std::cerr << "Unknown option. Try \"pstome -h\"." << std::endl;
                        return (248);
                }
                if (ecode==pstome::INFILENOTFOUND){
                        std::cerr << "Input file not found." << std::endl;
                        return (247);
                }
                if (ecode==pstome::PSSELECTNOTFOUND){
                        std::cerr << "psselect executable not found." << std::endl;
                        return (246);
                }
                if (ecode==pstome::PSNUPNOTFOUND){
                        std::cerr << "psnup executable not found." << std::endl;
                        return (245);
                }
                if (ecode==pstome::EMPTYPAGELIST){
                        std::cerr << "Internal fault. Try to generate file from empty pagelist." << std::endl;
                        return (244);
                }
                if (ecode==pstome::UNABLETOEXECUTE){
                        std::cerr << "Unable to execute." << std::endl;
                        return (243);
                }
                if (ecode==pstome::PDF2PSNOTFOUND){
                        std::cerr << "pdf2ps executable not found." << std::endl;
                        return (242);
                }
        }
        return 0;
}

